/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "actiontreeitem.h"
#include "actiontreeitem.moc"

#include <QtCore/QDebug>

namespace NSCore {

	ActionTreeItem::ActionTreeItem( QString n, QTreeItem* p, QString iconpath )
		: QTreeItem( n, p )
		, _guihint( 3 )
		, _iconpath( iconpath )
	{ }
	ActionTreeItem::ActionTreeItem( QString n, QTreeItem* p, QObject* o, const char* s )
		:QTreeItem( n, p )
		, _guihint( 3 )
	{
		connect( this, SIGNAL( activated() ), o, s );
	}
	ActionTreeItem::ActionTreeItem( QString n, QTreeItem* p, QString i, QObject* o, const char* s )
		: QTreeItem( n, p )
		, _guihint( 3 )
		, _iconpath( i )
	{
		connect( this, SIGNAL( activated() ), o, s );
	}

	ActionTreeItem::ActionTreeItem( ActionTreeItem& item, QTreeItem* p )
		: QTreeItem( item.objectName(), p )
	{
		qWarning() << "ActionTreeItem::ActionTreeItem(" << &item << "," << p << ") [not tested]";
		foreach( QString s, item.childlist() ) {
			ActionTreeItem* tmp = item.child<ActionTreeItem>( s );
			if ( tmp != 0 )
				new ActionTreeItem( *tmp, this );
		}
	}
	ActionTreeItem::~ActionTreeItem() {
		//qDebug() << "ActionTreeItem::~ActionTreeItem()" << path() << " " << objectName();
	}

	void ActionTreeItem::exec() {
		emit activated();
	}

	Qt::ItemFlags ActionTreeItem::guiFlags() const {
		return QTreeItem::guiFlags() | Qt::ItemIsEditable | Qt::ItemIsSelectable;
	}
	QStringList ActionTreeItem::guiActions() const {
		return QTreeItem::guiActions() << "Execute";
	}
	void ActionTreeItem::guiExecAction( QString action, GuiServer_Interface* gui ) {
		//qDebug() << "ActionTreeItem::guiExecAction(" << action << "," << gui <<")";
		if ( action == "Execute" ) {
			exec();
		} else
			QTreeItem::guiExecAction( action, gui );
	}
	QVariant ActionTreeItem::guiIdentifier( int role ) const {
		switch ( role ) {
			case Qt::ToolTipRole:
				return _tooltip;
				break;
			case Qt::DecorationRole:
				return _iconpath;
				break;
			default:
				return QTreeItem::guiIdentifier( role );
		}
		return QVariant();
	}

	QDomNode ActionTreeItem::saveToXML( QDomDocument doc ) {
		QDomElement node = doc.createElement( "action" );
		node.setAttribute( "name", objectName() );
		return node;
	}
	bool ActionTreeItem::restoreFromXML( QDomNode node ) {
		if ( node.nodeName() == "action" && node.isElement() ) {
			QDomElement elem = node.toElement();
			if ( elem.attribute( "name" ) == objectName() )
				return true;
		}
		return false;
	}

};

