/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef CACHINGGUISERVER_H
#define CACHINGGUISERVER_H

#include <guiserver_interface.h>

namespace NSCore {

/**
 * @brief Caching proxy for GuiServer
 *
 * When using the same tree of objects in multiple threads, the same questions
 * for the gui will be asked. This class caches the results of the first
 * question and gives the result to the subsequent questions.
 *
 * Which means when one question is asked two times in a row (by multiple
 * objects in different threads on user interaction) the user will be asked only
 * once.
 */
class CachingGuiServer : public GuiServer_Interface
{
	public:
		CachingGuiServer( GuiServer_Interface* );

		void message( const QString& title, const QString& text ) const;
		bool messageYesNo( const QString& title, const QString& text ) const;
		bool messageOkCancel( const QString& title, const QString& text ) const;
		double getDouble( const QString& title, const QString& text, double, double, double ) const;
		int getInt( const QString&, const QString&, int, int, int ) const;
		QString getString( const QString& title, const QString& label, const QString& initvalue ) const;
		QString getItem( const QString& title, const QString& label, const QStringList& list ) const;
		QString getItem( const QString& title, const QString& desc, const QList<ListItem> items ) const;
		QString getFileName( const QString& title, const QString& dir, const QString& filter, const FileType& );

	private:
		GuiServer_Interface* _gui;

		mutable struct {
			uint last;
			QString title, text;
		} _msg_cache;
		mutable struct {
			uint last;
			QString title, text;
			bool ret;
		} _msgOkCancel_cache;
		mutable struct {
			uint last;
			QString title, text;
			double value, min, max, ret;
		} _getDouble_cache;
		mutable struct {
			uint last;
			QString title, text;
			int value, min, max, ret;
		} _getInt_cache;
		mutable struct {
			uint last;
			QString title, text, value, ret;
		} _getString_cache;
		mutable struct {
			uint last;
			QString title, text, ret;
			QStringList list;
		} _getItemStringList_cache;
		mutable struct {
			uint last;
			QString title, text, ret;
			QList<ListItem> list;
		} _getItem_cache;
		mutable struct {
			uint last;
			QString title, dir, filter, ret;
			FileType type;
		} _getFileName_cache;
};

}; // NSCore

#endif // CACHINGGUISERVER_H
