/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "namedqvarianttreeitem.h"

#include <QtCore/QDebug>

namespace NSCore {

	NamedQVariantTreeItem::NamedQVariantTreeItem( QString n, QVariant v, AbstractTreeItem* p )
		: AbstractTreeItem( n, p )
		, QVariant( v )
	{
		//qDebug() << "NamedQVariantTreeItem::NamedQVariantTreeItem(" << n << "," << v << "," << p << ")";
	}
	NamedQVariantTreeItem::NamedQVariantTreeItem( NamedQVariantTreeItem& item, AbstractTreeItem* p )
		: AbstractTreeItem( item, p )
		, QVariant( item )
	{
		qDebug() << "NamedQVariantTreeItem::NamedQVariantTreeItem( NamedQVariantTreeItem&" << item << "," << p << ")\n ! Copy-constructor !";
		/*foreach( QString s, item.childlist() ) {
			NamedQVariantTreeItem* tmp = dynamic_cast<NamedQVariantTreeItem*>( item.child( s ) );
			if ( tmp != 0 )
				new NamedQVariantTreeItem( *tmp, this );
		}*/
	}
	NamedQVariantTreeItem::NamedQVariantTreeItem( NamedQVariantTreeItem* item, AbstractTreeItem* p )
		: AbstractTreeItem( *( item ), p )
		, QVariant( item )
	{
		qDebug() << "NamedQVariantTreeItem::NamedQVariantTreeItem( NamedQVariantTreeItem*" << item << "," << p << ")\n ! Copy-constructor !";
		/*foreach( QString s, item.childlist() ) {
			NamedQVariantTreeItem* tmp = dynamic_cast<NamedQVariantTreeItem*>( item.child( s ) );
			if ( tmp != 0 )
				new NamedQVariantTreeItem( *tmp, this );
		}*/
	}

	NamedQVariantTreeItem::~NamedQVariantTreeItem() {
	}

	NamedQVariantTreeItem* NamedQVariantTreeItem::getQVariant( QString n ) {
		//qDebug() << "NamedQVariantTreeItem::getQVariant(" << n << ")";
		NamedQVariantTreeItem* parameter = getNamedChild<NamedQVariantTreeItem*>( n );
		if ( parameter ) {
			//qDebug() << " found a valid QVariant" << parameter;
			return parameter;
		}
		return 0;
	}

	void NamedQVariantTreeItem::addQVariant( QString path, QVariant value ) {
		//qDebug() << "NamedQVariantTreeItem::addQVariant(" << path << "," << value << ")";
		QString parentpath = path.section( "/", 0, -2 );
		NamedQVariantTreeItem* p = getNamedChild<NamedQVariantTreeItem*>( parentpath );
		if ( parentpath != "" && p == 0 ) {
			QString firstpath = parentpath.section( "/", 0, 0 );
			NamedQVariantTreeItem* item = getQVariant( firstpath );
			if ( !item )
				addQVariant( firstpath, QVariant::Invalid );
			item = getQVariant( firstpath );
			if ( item )
				item->addQVariant( path.section( "/", 1, -1 ), value );
			else
				qWarning() << "! Warning: couldn't create child!";
		} else {
			if ( parentpath == "" )
				p = this;
			new NamedQVariantTreeItem( path.section( "/", -1, -1 ), value, p );
		}
	}

	void NamedQVariantTreeItem::setValue( QVariant v ) {
		//qDebug() << "NamedQVariantTreeItem::setValue(" << v << ") own name" << name();
		if ( type() == v.type() ) {
			//qDebug() << " Both types are" << QVariant::typeToName( type() );
			switch( type() ) {
				case QVariant::Double:
					QVariant::setValue( v.toDouble() );
					break;
				case QVariant::Int:
					QVariant::setValue( v.toInt() );
					break;
				case QVariant::String:
					QVariant::setValue( v.toString() );
					break;
				case QVariant::Invalid:
					break;
				default:
					qWarning() << "NamedQVariantTreeItem::setValue(" << v << ") Unhandled type. Did not set the value!";
			}
		}
	}

	QDataStream& operator<<( QDataStream& stream, const NamedQVariantTreeItem* item ) {
		if ( item != 0 ) {
			const QVariant* tmp = item;
			stream << item->name().toUtf8() << *tmp << item->childs();
			foreach( QString n, item->childlist() )
				stream << item->child<NamedQVariantTreeItem*>( n );
		}
		return stream;
	}
	QDataStream& operator>>( QDataStream& stream, NamedQVariantTreeItem* item ) {
		QByteArray a;
		stream >> a;
		QString name = QString::fromUtf8( a );
		QVariant value( stream );
		int childs;
		stream >> childs;
		NamedQVariantTreeItem* tmp = 0;
		if ( name == item->name() )
			tmp = item;
		else
			tmp = item->child<NamedQVariantTreeItem*>( name );
		if ( tmp )
			tmp->setValue( value );
		else {
			item->addQVariant( name, value );
			tmp = item->child<NamedQVariantTreeItem*>( name );
		}
		for ( int i=0; i<childs; ++i )
			stream >> tmp;
		return stream;
	}

};

