/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DISKIOBUFFERS_BIN_H
#define DISKIOBUFFERS_BIN_H

#include <QtCore/QFile>
#include <QtCore/QIODevice>
#include <QtCore/QDataStream>

#include <abstractdiskio.h>

namespace NSDiskIO {

	class EPOSBinWriter : public AbstractWriter, protected AbstractDiskIO<QDataStream>
	{
		Q_OBJECT
		public:
			EPOSBinWriter( QObject* =0 );
			~EPOSBinWriter();

			void writeDataSet( NSCore::AcqisitionDataSet& data );

			QString ending() const { return "eposbinfile"; }

		public slots:
			void setFile( QString filename );

		private slots:
			void stripChanged( bool );

		private:
			bool _strip;

	};

	class EPOSBinReader : public AbstractReader, protected AbstractDiskIO<QDataStream>
	{
		Q_OBJECT
		public:
			EPOSBinReader( QObject* =0 );
			~EPOSBinReader();

			void reset();
			bool atEnd() const { if ( _stream ) return _stream->atEnd(); return true; }

			QString filter() const { return "EPOS binary file (*.eposbinfile)"; }

			void setFile( QString );
			QString filename() const;

			void readSingleDataSet();

		private:
			int _headertoskip;
			/**
			 * @brief version of the file to read
			 *
			 * 0 is version 1 ( without the magic-header ) and version 2 ( with the magic-header %EPOSBIN2% )
			 * 1 is version 3 ( with magic-header %EPOSBIN3% )
			 */
			int _fileversion;
	};

	class PragReader : public AbstractReader, protected AbstractDiskIO<QDataStream>
	{
		Q_OBJECT
		public:
			PragReader( QObject* =0 );
			~PragReader();

			void reset() { if ( _stream ) _stream->device()->reset(); }
			bool atEnd() const { if ( _stream ) return _stream->atEnd(); return true; }

			QString filter() const { return "AcqirisLive Binary data (*.*)"; }

			void setFile( QString );
			QString filename() const;

			void readSingleDataSet();
	};

};

#endif // DISKIOBUFFERS_BIN_H

