/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "filterexpression.h"

#include <QtCore/QDebug>

namespace NSEvaluation {

FilterExpression::FilterExpression( NSCore::QTreeItem* p ) : NSCore::MathTreeRoot( p ), filter( false ) {
	//qDebug() << "FilterExpression::FilterExpression()";
	setObjectName( "Filter" );
}

double FilterExpression::value( NSCore::AcqisitionDataSet& data ) {
	if ( !filter )
		return 1;

	//qDebug() << "FilterExpression::value()";

	double ret = 0;
	foreach( NSCore::MathExpression* item, childobjects<NSCore::MathExpression>() ) {
		ret += item->value( data );
	}
	//qDebug() << " will return" << ret;
	return ret;
}

void FilterExpression::guiSetValue( QVariant n ) {
	if ( n.canConvert( QVariant::Bool ) )
		filter = n.toBool();
}

QDomNode FilterExpression::saveToXML( QDomDocument doc ) {
	//qDebug() << "FilterExpression::saveToXML()";
	QDomElement elem = doc.createElement( "filterpath" );
	elem.setAttribute( "filter", filter?"true":"false" );
	return elem;
}
bool FilterExpression::restoreFromXML( QDomNode node ) {
	//qDebug() << "FilterExpression::restoreFromXML(" << node.nodeName() << ")";
	if ( node.nodeName() == "filterpath" ) {
		filter = ( node.toElement().attribute( "filter", "false" ) == "true" );
		return true;
	}
	return false;
}

};

