/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/
#ifndef CHARTVIEW_TEST_H
#define CHARTVIEW_TEST_H

#include <QtGui/QtGui>
#include <chartview.h>
#include <flickcharm.h>

class ChartViewTest : public QWidget
{
	Q_OBJECT

	public:
		ChartViewTest( QWidget* p ) : QWidget( p ) {
			model = new QStandardItemModel( 0,0, this );
			QStringList rows;
			for ( int i=-19; i<=100; ++i ) {
				rows << QString( "%1" ).arg( i );
			}
			model->setVerticalHeaderLabels( rows );

			FlickCharm* flick = new FlickCharm( this );

			QGridLayout* layout = new QGridLayout( this );
			QTabWidget* tabs = new QTabWidget( this );
			tableview = new QTableView();
			tableview->setModel( model );
			tabs->addTab( tableview, "Table" );
			chartview = new ChartView();
			flick->activateOn( chartview );
			chartview->setModel( model );
			tabs->addTab( chartview, "Chart" );
			layout->addWidget( tabs, 0, 0, 4, 1 );
			btnAddSinus = new QPushButton( "Add Sinus", this );
			connect( btnAddSinus, SIGNAL( clicked() ), this, SLOT( addSinus() ) );
			layout->addWidget( btnAddSinus, 0, 1 );
			btnAddLinear = new QPushButton( "Add Linear", this );
			connect( btnAddLinear, SIGNAL( clicked() ), this, SLOT( addLinear() ) );
			layout->addWidget( btnAddLinear, 1, 1 );
			btnClear = new QPushButton( "Clear Data", this );
			connect( btnClear, SIGNAL( clicked() ), this, SLOT( clearData() ) );
			layout->addWidget( btnClear, 2, 1 );
		}
		~ChartViewTest() {
		}

	public slots:
		void addSinus() {
			//qDebug() << "ChartViewTest::addData()";
			//qDebug() << " rows =" << model->rowCount() << " cols =" << model->columnCount();
			int col = model->columnCount();
			model->appendColumn( QList<QStandardItem*>() );
			for ( int i=0; i<model->rowCount(); ++i ) {
				QModelIndex index = model->index( i, col );
				model->setData( index, 5 / (col+1.0) * sin( i * 1.0 / 20*M_PI ) + 6);
			}
			//qDebug() << " rows =" << model->rowCount() << " cols =" << model->columnCount();
		}
		void addLinear() {
			//qDebug() << "ChartViewTest::addData()";
			//qDebug() << " rows =" << model->rowCount() << " cols =" << model->columnCount();
			int col = model->columnCount();
			model->appendColumn( QList<QStandardItem*>() );
			for ( int i=0; i<model->rowCount(); ++i ) {
				QModelIndex index = model->index( i, col );
				model->setData( index, (i-19.0) / 20 * (col+1.0) );
			}
			//qDebug() << " rows =" << model->rowCount() << " cols =" << model->columnCount();
		}

		void clearData() {
			model->removeColumn( model->columnCount()-1 );
			/*model->clear();
			QStringList rows;
			for ( int i=-19; i<=100; ++i ) {
				rows << QString( "%1" ).arg( i );
			}
			model->setVerticalHeaderLabels( rows );*/
		}

	public:
		QStandardItemModel* model;
		QTableView* tableview;
		ChartView* chartview;
		QPushButton *btnAddSinus, *btnAddLinear, *btnClear;
};

#endif // CHARTVIEW_TEST_H

