/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef FANCYLISTVIEW_H
#define FANCYLISTVIEW_H

#include <guiserver_interface.h>

#include <QtCore/QObject>
#include <QtGui/QDialog>
#include <QtGui/QItemDelegate>
#include <QtGui/QStandardItemModel>
#include <QtGui/QAbstractItemView>

class QWidget;
class QTreeWidgetItem;

class SettingsContainer;

class Ui_FancyListDialog;

class FancyListDialog : public QDialog
{
	Q_OBJECT
	public:
		FancyListDialog( const QString& title, const QString& desc, QWidget* );
		~FancyListDialog();

		void addItems( QList<NSCore::GuiServer_Interface::ListItem> items );

		QString currentItem;

	private slots:
		void on_search_textEdited( const QString& );

		void currentChanged( const QModelIndex&, const QModelIndex& );

	private:
		Ui_FancyListDialog* d;
		QStandardItemModel* _model;
		QList<QStandardItem*> _categories;
		QList<QStandardItem*> _items;
};

class FancyListDialogDelegate : public QItemDelegate
{
	public:
		FancyListDialogDelegate( QObject* );

		void paint( QPainter*, const QStyleOptionViewItem&, const QModelIndex& ) const;

		QSize sizeHint( const QStyleOptionViewItem&, const QModelIndex& ) const;
};

class FancyListView : public QAbstractItemView
{
	Q_OBJECT
	public:
		FancyListView( QWidget* );

		QRect visualRect( const QModelIndex& ) const;
		QRegion visualRegionForSelection( const QItemSelection& ) const { return QRegion(); }

		void scrollTo( const QModelIndex&, QAbstractItemView::ScrollHint );

		QModelIndex indexAt( const QPoint& ) const;
		QModelIndex moveCursor( QAbstractItemView::CursorAction, Qt::KeyboardModifiers );

		int horizontalOffset() const { return 0; }
		int verticalOffset() const { return 0; }

		bool isIndexHidden( const QModelIndex& ) const { return false; }

		void setSelection( const QRect&, QFlags<QItemSelectionModel::SelectionFlag> ) {}

	public slots:
		void setModel( QAbstractItemModel* );

	protected slots:
		void dataChanged( const QModelIndex&, const QModelIndex& );
		void rowsInserted( const QModelIndex&, int, int );

	private:
		QList<SettingsContainer*> containers;
};

#endif // FANCYLISTVIEW_H

