/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "acqiristabs_v2_widget.h"
#include "acqiristabs_v2_widget.moc"

#include "acqiristabs_v2.h"

#include <QtCore/QDebug>
#include <inttofloat.h>
#include <parameter_gui.h>
#include "akhline.h"

using namespace NSCore;

AcqirisTabsV2Widget::AcqirisTabsV2Widget( NSAcqiris::Digitizer* d, QWidget* p ) : QFrame( p ), _digitizer( d ) {
	_base = new Ui::AcqirisTabsV2();
	qDebug() << "AcqirisTabsV2Widget::AcqirisTabsV2Widget( NSAcqiris::Digitizer* " << d << ", QWidget* " << p << " )";
	_base->setupUi( this );
	connect( _base->btnStart, SIGNAL( clicked() ), _digitizer, SLOT( startAcqisition() ) );
	connect( _base->btnStop, SIGNAL( clicked() ), _digitizer, SLOT( stopAcqisition() ) );
	connect( _base->btnCalibrate, SIGNAL( clicked() ), _digitizer, SLOT( calibrate() ) );
	connect( _base->btnSingleRun, SIGNAL( toggled( bool ) ), _digitizer, SLOT( singleShot( bool ) ) );
	//btnSingleRun->toggle();

	NamedParameterTreeItemPointer config( _digitizer->config() );
	//config->debugPrint();

	/// General parameters
	_base->timeout->setValue( config[ "timeout" ]->toInt() );
	connect( _base->timeout, SIGNAL( valueChanged( int ) ),
	         config[ "timeout" ], SLOT( setValue( int ) ) );
	connect( config[ "timeout" ], SIGNAL( valueChanged( int ) ),
	         _base->timeout, SLOT( setValue( int ) ) );

	_base->checkboxForceTrigger->setChecked( config[ "forcetrigger" ]->toBool() );
	connect( _base->checkboxForceTrigger, SIGNAL( toggled( bool ) ),
	         config[ "forcetrigger" ], SLOT( setValue( bool ) ) );
	connect( config[ "forcetrigger" ], SIGNAL( valueChanged( bool ) ),
	         _base->checkboxForceTrigger, SLOT( setChecked( bool ) ) );

	_base->samplingrate->setText( config[ "horizontal/samplinginterval" ]->toString() );
	// Works because the string can be converted to a number directly...
	connect( _base->samplingrate, SIGNAL( textChanged( const QString& ) ),
	         config[ "horizontal/samplinginterval" ], SLOT( setValue( const QString & ) ) );
	connect( config[ "horizontal/samplinginterval" ], SIGNAL( valueChanged( const QString& ) ),
	         _base->samplingrate, SLOT( setText( const QString& ) ) );

	_base->numberofSamples->setValue( config[ "horizontal/samples" ]->toInt() );
	connect( _base->numberofSamples, SIGNAL( valueChanged( int ) ),
	         config[ "horizontal/samples" ], SLOT( setValue( int ) ) );
	connect( config[ "horizontal/samples" ], SIGNAL( valueChanged( int ) ),
	         _base->numberofSamples, SLOT( setValue( int ) ) );
	_base->numberofSamples->setRange( config[ "horizontal/samples" ]->min().toInt(), config[ "horizontal/samples" ]->max().toInt() );

	_base->numberOfSegments->setValue( config[ "horizontal/segments" ]->toInt() );
	connect( _base->numberOfSegments, SIGNAL( valueChanged( int ) ),
	         config[ "horizontal/segments" ], SLOT( setValue( int ) ) );
	connect( config[ "horizontal/segments" ], SIGNAL( valueChanged( int ) ),
	         _base->numberOfSegments, SLOT( setValue( int ) ) );
	_base->numberOfSegments->setRange( config[ "horizontal/segments" ]->min().toInt(), config[ "horizontal/segments" ]->max().toInt() );

	connect( _base->spinboxTriggerDelay, SIGNAL( valueChanged( int ) ),
	         config[ "horizontal/delaytime" ], SLOT( setValue( int ) ) );
	connect( config[ "horizontal/delaytime" ], SIGNAL( valueChanged( int ) ),
	         _base->spinboxTriggerDelay, SLOT( setValue( int ) ) );

	_base->comboboxTriggerSource->addItems( _digitizer->triggersources() );
	connect( _base->comboboxTriggerSource, SIGNAL( activated( const QString& ) ),
	         _digitizer, SLOT( triggersource( const QString& ) ) );
	//         config->getOption( "trigger/channel" ), SLOT( value( int ) ) );

	connect( _base->spinboxTriggerLevelIntern, SIGNAL( valueChanged( double ) ),
	         config[ "trigger/levelintern" ], SLOT( setValue( double ) ) );
	connect( config[ "trigger/levelintern" ], SIGNAL( valueChanged( double ) ),
	         _base->spinboxTriggerLevelIntern, SLOT( setValue( double ) ) );
	connect( _base->spinboxTriggerLevelExtern, SIGNAL( valueChanged( double ) ),
	         config[ "trigger/levelextern" ], SLOT( setValue( double ) ) );
	connect( config[ "trigger/levelextern" ], SIGNAL( valueChanged( double ) ),
	         _base->spinboxTriggerLevelExtern, SLOT( setValue( double ) ) );

	_base->comboboxTriggerSlope->addItems( _digitizer->triggerslopes() );
	connect( _base->comboboxTriggerSlope, SIGNAL( activated( int ) ),
	         config[ "trigger/slope" ], SLOT( setValue( int ) ) );
	connect( config[ "trigger/slope" ], SIGNAL( valueChanged( int ) ),
	         _base->comboboxTriggerSlope, SLOT( setCurrentIndex( int ) ) );

	_base->comboboxTriggerCoupling->addItems( _digitizer->triggercouplings() );
	connect( _base->comboboxTriggerCoupling, SIGNAL( activated( int ) ),
	         config[ "trigger/coupling" ], SLOT( setValue( int ) ) );
	connect( config[ "trigger/coupling" ], SIGNAL( valueChanged( int ) ),
	         _base->comboboxTriggerCoupling, SLOT( setCurrentIndex( int ) ) );

	/// state-LED
	/*_base->ledState->setOff();
	connect( _digitizer, SIGNAL( acquisitionStarted() ), _base->ledState, SLOT( setOn() ) );
	connect( _digitizer, SIGNAL( acquisitionStopped() ), _base->ledState, SLOT( setOff() ) );
	connect( _digitizer, SIGNAL( acquisitionStoppedTimeout() ), _base->ledState, SLOT( setError() ) );*/

	_base->comboBoxLED->addItems( _digitizer->LEDColors() );
	connect( _base->comboBoxLED, SIGNAL( activated( const QString& ) ), _digitizer, SLOT( setLEDColor( const QString& ) ) );

	/// Channels
	QWidget* tmp =0;
	for ( int i=0; i<_base->tab->count(); ++i ) {
		if ( _base->tab->tabText( i ) == "Channels" )
			tmp = _base->tab->widget( i );
	}
	QBoxLayout *layout = new QVBoxLayout( tmp );
	for ( int i=0; i<_digitizer->channels() && tmp!=0; ++i ) {
		QWidget* bla = new ParameterGui( _digitizer->config()->getParameter( QString( "Channel %1" ).arg( i+1 ) ), tmp );
		layout->addWidget( bla );
	}
	/// Channels
	tmp =0;
	for ( int i=0; i<_base->tab->count(); ++i ) {
		if ( _base->tab->tabText( i ) == "Ext. Trigger" )
			tmp = _base->tab->widget( i );
	}
	layout = new QVBoxLayout( tmp );
	for ( int i=0; i<_digitizer->externaltriggers() && tmp!=0; ++i ) {
		QWidget* bla = new ParameterGui( _digitizer->config()->getParameter( QString( "Channel %1" ).arg( -i-1 ) ), tmp );
		layout->addWidget( bla );
	}
	// Just a simple spacer...
	QWidget *tmp2 = new QWidget( tmp );
	tmp2->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
	layout->addWidget( tmp2 );
}
AcqirisTabsV2Widget::~AcqirisTabsV2Widget() {
	qDebug() << "AcqirisTabsV2Widget::~AcqirisTabsV2Widget()";
}

